@@ -9,7 +9,8 @@ from typing import Any, Container, DefaultDict, Dict, List, Set, Union
 import httpx
 import yaml
 from github import Github
-from pydantic import BaseModel, BaseSettings, SecretStr
+from pydantic import BaseModel, SecretStr
+from pydantic_settings import BaseSettings
 
 github_graphql_url = "https://api.github.com/graphql"
 questions_category_id = "MDE4OkRpc2N1c3Npb25DYXRlZ29yeTMyMDAxNDM0"
@@ -352,7 +353,6 @@ class SponsorsResponse(BaseModel):
 
 class Settings(BaseSettings):
     input_token: SecretStr
-    input_standard_token: SecretStr
     github_repository: str
     httpx_timeout: int = 30
 
@@ -381,12 +381,17 @@ def get_graphql_response(
         logging.error(response.text)
         raise RuntimeError(response.text)
     data = response.json()
+    if "errors" in data:
+        logging.error(f"Errors in response, after: {after}, category_id: {category_id}")
+        logging.error(data["errors"])
+        logging.error(response.text)
+        raise RuntimeError(response.text)
     return data
 
 
 def get_graphql_issue_edges(*, settings: Settings, after: Union[str, None] = None):
     data = get_graphql_response(settings=settings, query=issues_query, after=after)
-    graphql_response = IssuesResponse.parse_obj(data)
+    graphql_response = IssuesResponse.model_validate(data)
     return graphql_response.data.repository.issues.edges
 
 
@@ -401,19 +406,19 @@ def get_graphql_question_discussion_edges(
         after=after,
         category_id=questions_category_id,
     )
-    graphql_response = DiscussionsResponse.parse_obj(data)
+    graphql_response = DiscussionsResponse.model_validate(data)
     return graphql_response.data.repository.discussions.edges
 
 
 def get_graphql_pr_edges(*, settings: Settings, after: Union[str, None] = None):
     data = get_graphql_response(settings=settings, query=prs_query, after=after)
-    graphql_response = PRsResponse.parse_obj(data)
+    graphql_response = PRsResponse.model_validate(data)
     return graphql_response.data.repository.pullRequests.edges
 
 
 def get_graphql_sponsor_edges(*, settings: Settings, after: Union[str, None] = None):
     data = get_graphql_response(settings=settings, query=sponsors_query, after=after)
-    graphql_response = SponsorsResponse.parse_obj(data)
+    graphql_response = SponsorsResponse.model_validate(data)
     return graphql_response.data.user.sponsorshipsAsMaintainer.edges
 
 
@@ -496,21 +501,25 @@ def get_discussions_experts(settings: Settings):
 
 
 def get_experts(settings: Settings):
-    (
-        issues_commentors,
-        issues_last_month_commentors,
-        issues_authors,
-    ) = get_issues_experts(settings=settings)
+    # Migrated to only use GitHub Discussions
+    # (
+    #     issues_commentors,
+    #     issues_last_month_commentors,
+    #     issues_authors,
+    # ) = get_issues_experts(settings=settings)
     (
         discussions_commentors,
         discussions_last_month_commentors,
         discussions_authors,
     ) = get_discussions_experts(settings=settings)
-    commentors = issues_commentors + discussions_commentors
-    last_month_commentors = (
-        issues_last_month_commentors + discussions_last_month_commentors
-    )
-    authors = {**issues_authors, **discussions_authors}
+    # commentors = issues_commentors + discussions_commentors
+    commentors = discussions_commentors
+    # last_month_commentors = (
+    #     issues_last_month_commentors + discussions_last_month_commentors
+    # )
+    last_month_commentors = discussions_last_month_commentors
+    # authors = {**issues_authors, **discussions_authors}
+    authors = {**discussions_authors}
     return commentors, last_month_commentors, authors
 
 
@@ -600,8 +609,8 @@ def get_top_users(
 if __name__ == "__main__":
     logging.basicConfig(level=logging.INFO)
     settings = Settings()
-    logging.info(f"Using config: {settings.json()}")
-    g = Github(settings.input_standard_token.get_secret_value())
+    logging.info(f"Using config: {settings.model_dump_json()}")
+    g = Github(settings.input_token.get_secret_value())
     repo = g.get_repo(settings.github_repository)
     question_commentors, question_last_month_commentors, question_authors = get_experts(
         settings=settings
