@@ -8,7 +8,7 @@ app = FastAPI()
 @app.get("/items/")
 async def read_items(
     q: Union[str, None] = Query(
-        default=None, min_length=3, max_length=50, regex="^fixedquery$"
+        default=None, min_length=3, max_length=50, pattern="^fixedquery$"
     )
 ):
     results = {"items": [{"item_id": "Foo"}, {"item_id": "Bar"}]}
