@@ -7,10 +7,10 @@ app = FastAPI()
 
 
 class Item(BaseModel):
-    name: str = Field(example="Foo")
-    description: Union[str, None] = Field(default=None, example="A very nice Item")
-    price: float = Field(example=35.4)
-    tax: Union[float, None] = Field(default=None, example=3.2)
+    name: str = Field(examples=["Foo"])
+    description: Union[str, None] = Field(default=None, examples=["A very nice Item"])
+    price: float = Field(examples=[35.4])
+    tax: Union[float, None] = Field(default=None, examples=[3.2])
 
 
 @app.put("/items/{item_id}")
