@@ -4,7 +4,11 @@ from pathlib import Path
 
 import pytest
 
+from ...utils import needs_pydanticv1
 
+
+# TODO: pv2 add version with Pydantic v2
+@needs_pydanticv1
 def test_testing_dbs(tmp_path_factory: pytest.TempPathFactory):
     tmp_path = tmp_path_factory.mktemp("data")
     cwd = os.getcwd()
